#include <stdio.h>
#include <stdlib.h>
#include "playitsam.h"

/*
 * Cut list processing for Playitsam.
 */

extern int allow_playback;


/* We keep cuts on a singly-linked list ordered by the
 * chunk # of the start of the cut. Yes, we don't deal
 * with adjacent or overlapping cuts. Perhaps you would
 * like to add the code for that?!
 */

struct cutlist {
  int start;
  int end;
  struct cutlist *next;
};

static struct cutlist *Cuthead=NULL;	/* We keep the head of the list here */

/* When the user starts a cut, we cannot add a new node to the
 * list immediately. Instead, we record the start here.
 */
static int pending_cutstart=1;

void add_cut_start(int start)
{
  pending_cutstart=start;
}

/* Add a new cut to the list, and return the start chunk #,
 * or if there is some error, return -1.
 */
int add_cut_end(int end)
{
  struct cutlist *this, *S, *L;
  int start=pending_cutstart;

  if (pending_cutstart==0) return(-1);
  if (end < pending_cutstart) return(-1);
  pending_cutstart=0;

  this= (struct cutlist *)malloc(sizeof(struct cutlist));
  if (this==NULL) return(-1);

  this->start= start; this->end= end; this->next=NULL;

  if (Cuthead==NULL) { Cuthead=this; return(start); }

  if (Cuthead->start > this->start) { 
    this->next= Cuthead; Cuthead= this; return(start);
  }

  for (L=S=Cuthead; S; L=S, S=S->next) {
    if (S->next && (S->next->start >= start) &&
				(S->start <= start)) {
	this->next= S->next; S->next= this; return(start);
    }
  }

  /* If we get here, then we fell off the end. Fortunately, L holds
   * the last valid pointer.
   */
   L->next= this; return(start);
}

/* Find the cut starting at this chunk, delete it
 * and return 0, or if there is some error, return -1.
 */
int kill_cut_start(int start)
{
  struct cutlist *S, *L;

  for (L=S=Cuthead; S; L=S, S=S->next) {
    if (S->start == start) {
      if (Cuthead==S) { Cuthead= S->next; }
      else { L->next= S->next; }
      free(S); return(0);
    }
  }
  return(-1);
}

/* Find the cut ending at this chunk, delete it
 * and return 0, or if there is some error, return -1.
 */
int kill_cut_end(int end)
{
  struct cutlist *S, *L;

  for (L=S=Cuthead; S; L=S, S=S->next) {
    if (S->end == end) {
      if (Cuthead==S) { Cuthead= S->next; }
      else { L->next= S->next; }
      free(S); return(0);
    }
  }
  return(-1);
}

/* Find the last cut that we made, remove it from the
 * list, and return the start chunk #. If we cannot
 * do the removal, then return -1.
 */
int kill_last_cut(void)
{
 /* To do this, I need a second list ordered by time of
  * insertion. I will get around to it sometime.
  */
 return(-1);
}

void kill_all_cuts(void)
{
  struct cutlist *S, *L;

  for (S=Cuthead; S; ) { L=S; S=S->next; free(L); }
  Cuthead= NULL;
}

/* Given a chunk number, find out if this chunk is inside
 * a cut, outside, or on the edge. Return one of the IS_CUT_
 * constants. However, if allow_playback is 2, or
 * cutmore_bframes is 1, return quickly!
 */
int find_chunk_in_cutlist(int chunk, int cutmore_bframes)
{
  struct cutlist *S;

  if (allow_playback==2) return(IS_OUTSIDE_CUT);
  if (cutmore_bframes) return(IS_WANTPFRAME);

  for (S=Cuthead; S; S=S->next) {
    if (S->start == chunk) return(IS_CUT_START);
    if (S->end == chunk) return(IS_CUT_END);
    if ((S->start < chunk) && (S->end > chunk)) return(IS_INSIDE_CUT);
  }
  return(IS_OUTSIDE_CUT);
}

void list_cuts(void)
{
  struct cutlist *S;

  for (S=Cuthead; S; S=S->next) {
    fprintf(stderr, "  cut from chunk %d to %d\r\n", S->start, S->end);
  }
}
